// ScreenSizer Module
// $Author: Jeffrey J Weimer$
// SVN History: $Revision: 1228 $ on $Date: 2009-12-01 15:52:03 -0800 (Tue, 01 Dec 2009) $
//
// See the Help file for further information

#define English

#pragma rtGlobals=1
#pragma IgorVersion=6.10
#pragma version=2.3

#pragma ModuleName=ScreenSizer

#pragma hide=1

Static StrConstant thePackage="ScreenSizer"
Static StrConstant thePackageFolder="root:Packages:ScreenSizer"
Static StrConstant theProcedureFile = "ScreenSizer.ipf"
Static Constant thePackageVersion = 2.3
Static Constant hasHelp = 1

#ifdef English
Static StrConstant KillMeRequest="Do you really want to remove Screen Sizer?"
#endif
#ifdef Deutsch
Static StrConstant KillMeRequest="Moechten Sie verchlich Screen Sizer abloeschen?"
#endif

// Initialize
//  call to the function as ScreenSizer#Initialize(winXX)
//	winXX: 0 - do not change frame size on WinXX systems (default!)
//	winXX: 1 - use full frame size on WinXX systems
//   force (optional): 0 (default) - return if already set; 1 - force parameters to be updated

Static Function Initialize(winXX,[force])
	variable winXX, force
	
	variable lforce = 0
	
	if (!DataFolderExists(thePackageFolder))
	
		// initalize from clean start
	
		string cdf = GetDataFolder(1)
		
		NewDataFolder/O/S root:Packages
		NewDataFolder/O/S ScreenSizer
		variable/G scrwpxl, scrhpxl, scrwpnt, scrhpnt
		variable/G scrleft = 0, scrtop = 0, scrright = 100, scrbottom = 100
		SetDataFolder $cdf
		lforce = 1
	endif
	
	// check whether to force an update or not
	
	if (ParamIsDefault(force) && (!lforce))
		return 0
	endif
	if ((!force) && (!lforce))
		return 0
	endif
	if ((force!=1) && (!lforce))
		return 0
	endif
	
	// define the storage parameters

	NVAR scrwx = root:Packages:ScreenSizer:scrwpxl
	NVAR scrhx = root:Packages:ScreenSizer:scrhpxl
	NVAR scrwn = root:Packages:ScreenSizer:scrwpnt
	NVAR scrhn = root:Packages:ScreenSizer:scrhpnt
	
	// do Mac or Windows updates to parameters

	if (cmpstr(IgorInfo(2),"Macintosh")==0)

		string IgorStuff=""
		variable scr0,scr1,scr2
	
		IgorStuff=IgorInfo(0)
		scr0 = strsearch(IgorStuff,"RECT",0)
		scr1 = strsearch(IgorStuff,",",scr0+9)
		scr2 = strlen(IgorStuff)-2
		
		// values are in RELATIVE PIXELS (for tables and graphs)!!!!
		
		scrwx = str2num(IgorStuff[scr0+9,scr1-1])/100
		scrhx = str2num(IgorStuff[scr1+1,scr2])/100
	else

		// maximize frame on Windows
		
		if (winXX==1)
			MoveWindow/F 2, 2, 2, 2
		endif
		
		GetWindow kwFrameInner wsizeDC

		// values are in RELATIVE PIXELS (for panels)!!!!
		
		scrwx = (v_right-v_left)/100
		scrhx = (v_bottom-v_top)/100
	endif
	
	// values in RELATIVE POINTS (for tables and graphs)
	
	scrwn = scrwx*72/ScreenResolution
	scrhn = scrhx*72/ScreenResolution
	
	return 0
End

// Set Frame

Static Function SetFrame(left, top, right, bottom)
	variable left, top, right, bottom
	
	if (!ConfirmSet(left,top,right,bottom))
		return -1
	endif
	
	Initialize(0)
	
	NVAR scrleft = root:Packages:ScreenSizer:scrleft
	NVAR scrtop = root:Packages:ScreenSizer:scrtop
	NVAR scrright = root:Packages:ScreenSizer:scrright
	NVAR scrbottom = root:Packages:ScreenSizer:scrbottom
	
	scrleft = left
	scrtop = top
	scrright = right
	scrbottom = bottom
	return 0
end

// ReSet Frame

Static Function ReSetFrame()

	SetFrame(0,0,100,100)
	return 0
end

// Move Front Window

Static Function MoveFrontWindow(left, top, right, bottom)
	variable left, top, right, bottom
	
	if (!ConfirmSet(left,top,right,bottom))
		return -1
	endif
	
	Initialize(0)
	
	NVAR scrleft = root:Packages:ScreenSizer:scrleft
	NVAR scrtop = root:Packages:ScreenSizer:scrtop
	NVAR scrright = root:Packages:ScreenSizer:scrright
	NVAR scrbottom = root:Packages:ScreenSizer:scrbottom

	NVAR scrwpnt = root:Packages:ScreenSizer:scrwpnt
	NVAR scrhpnt = root:Packages:ScreenSizer:scrhpnt

	left=(scrwpnt*(scrleft + (scrright - scrleft)*left/100))
	top=(scrhpnt*(scrtop + (scrbottom-scrtop)*top/100))
	right=(scrwpnt*(scrleft + (scrright - scrleft)*right/100))
	bottom=(scrhpnt*(scrtop + (scrbottom-scrtop)*bottom/100))
	
	string name = WinName(0,(1+2+4+16+64+128+4096))
	if (WinType(name)==0)
		MoveWindow/P=$name left,top,right,bottom
	else
		MoveWindow left,top,right,bottom
	endif
	return 0
end

// Move A Window

Static Function MoveAWindow(left, top, right, bottom, name)
	variable left, top, right, bottom
	string name
	
	variable theWinType=WinType(name)
	
	if (theWinType==0)
		if (strlen(WinList(name,"","WIN:128"))==0)
			return -1
		endif
	endif	
	
	if (!ConfirmSet(left,top,right,bottom))
		return -1
	endif
	
	Initialize(0)
	
	NVAR scrleft = root:Packages:ScreenSizer:scrleft
	NVAR scrtop = root:Packages:ScreenSizer:scrtop
	NVAR scrright = root:Packages:ScreenSizer:scrright
	NVAR scrbottom = root:Packages:ScreenSizer:scrbottom

	NVAR scrwpnt = root:Packages:ScreenSizer:scrwpnt
	NVAR scrhpnt = root:Packages:ScreenSizer:scrhpnt

	left=(scrwpnt*(scrleft + (scrright - scrleft)*left/100))
	top=(scrhpnt*(scrtop + (scrbottom-scrtop)*top/100))
	right=(scrwpnt*(scrleft + (scrright - scrleft)*right/100))
	bottom=(scrhpnt*(scrtop + (scrbottom-scrtop)*bottom/100))
	
	if (theWinType==0)
		MoveWindow/P=$name left,top,right,bottom
	else
		MoveWindow/W=$name left,top,right,bottom
	endif
	
	return 0
end

// Move Windows
// type (optional): type number according to WinList and WinName, NOT WinType!
// kind (optional): graphs, tables, layouts, notebooks, panels, procedures, xops
// hidden (optional): 0 (default) - ignore hidden; 1 - also move hidden windows
// exclude (optional): string list of windows to exclude from move
// --> this command ignores panels, procedures, and XOP targets unless requested explicitly by type or kind
// --> this command always ignores the command window

Static Function MoveWindows(left, top, right, bottom,[type,kind,hidden,exclude])
	variable left, top, right, bottom
	variable type, hidden
	string kind, exclude
	
	if (!ConfirmSet(left,top,right,bottom))
		return -1
	endif
	
	if (ParamIsDefault(hidden))
		hidden=0
	endif
	
	if (ParamIsDefault(type) && ParamIsDefault(kind))
		type = 1 + 2 + 4 + 16
	else
		if (!ParamIsDefault(type))
			if (!((type & 2^0) || (type & 2^1) || (type & 2^2) || (type & 2^4) || (type & 2^6) || (type & 2^7) || (type & 2^12)))
				return -1
			endif
		else
			if (ParamIsDefault(kind))
				type = 1 + 2 + 4 + 16
			else
				strswitch(kind)
					case "graphs":
						type = 1
						break
					case "tables":
						type = 2
						break
					case "layouts":
						type = 4
						break
					case "notebooks":
						type = 16
						break
					case "panels":
						type = 64
						break
					case "procedures":
						type = 128
						break
					case "xops":
						type = 4096
						break
					default:
						return -1
				endswitch
			endif
		endif
	endif
		
	Initialize(0)
	
	NVAR scrleft = root:Packages:ScreenSizer:scrleft
	NVAR scrtop = root:Packages:ScreenSizer:scrtop
	NVAR scrright = root:Packages:ScreenSizer:scrright
	NVAR scrbottom = root:Packages:ScreenSizer:scrbottom

	NVAR scrwpnt = root:Packages:ScreenSizer:scrwpnt
	NVAR scrhpnt = root:Packages:ScreenSizer:scrhpnt
	
	variable ic,nt
	string theOne="", theList="", theType=""

	left=(scrwpnt*(scrleft + (scrright - scrleft)*left/100))
	top=(scrhpnt*(scrtop + (scrbottom-scrtop)*top/100))
	right=(scrwpnt*(scrleft + (scrright - scrleft)*right/100))
	bottom=(scrhpnt*(scrtop + (scrbottom-scrtop)*bottom/100))
	
	sprintf theType "WIN:%d" type
	theList=RemoveFromList("Procedure",WinList("*",";",theType))
	if (!hidden)
		theList = RemoveHiddenWindows(theList)
	endif
	if (!ParamIsDefault(exclude))
		theList = RemoveFromList(exclude,theList)
	endif
	nt = ItemsInList(theList)
	if (nt==0)
		return 0
	endif
	for (ic=0;ic<nt;ic+=1)
		theOne=StringFromList(ic,theList)
		if (WinType(theOne)==0)
			MoveWindow/P=$theOne left,top,right,bottom
		else
			MoveWindow/W=$theOne left,top,right,bottom
		endif
	endfor
	return 0
end

// Move Command Window

Static Function MoveCmdWindow(left, top, right, bottom)
	variable left, top, right, bottom
	
	if (!ConfirmSet(left,top,right,bottom))
		return -1
	endif
	
	Initialize(0)
	
	NVAR scrleft = root:Packages:ScreenSizer:scrleft
	NVAR scrtop = root:Packages:ScreenSizer:scrtop
	NVAR scrright = root:Packages:ScreenSizer:scrright
	NVAR scrbottom = root:Packages:ScreenSizer:scrbottom

	NVAR scrwpnt = root:Packages:ScreenSizer:scrwpnt
	NVAR scrhpnt = root:Packages:ScreenSizer:scrhpnt

	left=(scrwpnt*(scrleft + (scrright - scrleft)*left/100))
	top=(scrhpnt*(scrtop + (scrbottom-scrtop)*top/100))
	right=(scrwpnt*(scrleft + (scrright - scrleft)*right/100))
	bottom=(scrhpnt*(scrtop + (scrbottom-scrtop)*bottom/100))
	
	MoveWindow/C left,top,right,bottom
	return 0
end

// Size Front Window

Static Function SizeFrontWindow(width, height)
	variable width, height
	
	Initialize(0)

	NVAR scrwpnt = root:Packages:ScreenSizer:scrwpnt
	NVAR scrhpnt = root:Packages:ScreenSizer:scrhpnt
	
	variable right, bottom

	GetWindow kwTopWin, wsize
	right=V_left + (width*scrwpnt)
	bottom=V_top + (height*scrhpnt)

	string name = WinName(0,(1+2+4+16+64+128+4096))
	if (WinType(name)==0)
		MoveWindow/P=$name V_left,V_top,right,bottom
	else
		MoveWindow V_left,V_top,right,bottom
	endif

	return 0	
end

// Size a Window

Static Function SizeAWindow(width, height, name)
	variable width, height
	string name
	
	variable theWinType=WinType(name)
	
	if (theWinType==0)
		if (strlen(WinList(name,"","WIN:128"))==0)
			return -1
		endif
	endif	

	Initialize(0)

	NVAR scrwpnt = root:Packages:ScreenSizer:scrwpnt
	NVAR scrhpnt = root:Packages:ScreenSizer:scrhpnt
	
	variable right, bottom
	GetWindow $name wsize
	right=V_left + (width*scrwpnt)
	bottom=V_top + (height*scrhpnt)
	if (theWinType==0)
		MoveWindow/P=$name V_left,V_top,right,bottom
	else	
		MoveWindow/W=$name V_left,V_top,right,bottom
	endif
	return 0
	
end

// Size Windows
// type (optional): type number according to WinList and WinName, NOT WinType!
// kind (optional): graphs, tables, layouts, notebooks, panels, procedures, xops
// hidden (optional): 0 (default) - ignore hidden; 1 - also size hidden windows
// exclude (optional): string list of windows to exclude from size
// --> this command ignores panels, procedures, and XOP targets unless requested explicitly by type or kind
// --> this command always ignores the command window

Static Function SizeWindows(width, height, [type,kind,hidden,exclude])
	variable width, height, type, hidden
	string kind, exclude
	
	if (ParamIsDefault(hidden))
		hidden=0
	endif
	
	if (ParamIsDefault(type) && ParamIsDefault(kind))
		type = 1 + 2 + 4 + 16
	else
		if (!ParamIsDefault(type))
			if (!((type & 2^0) || (type & 2^1) || (type & 2^2) || (type & 2^4) || (type & 2^6) || (type & 2^7) || (type & 2^12)))
				return -1
			endif
		else
			if (ParamIsDefault(kind))
				type = 1 + 2 + 4 + 16
			else
				strswitch(kind)
					case "graphs":
						type = 1
						break
					case "tables":
						type = 2
						break
					case "layouts":
						type = 4
						break
					case "notebooks":
						type = 16
						break
					case "panels":
						type = 64
						break
					case "procedures":
						type = 128
						break
					case "xops":
						type = 4096
						break
					default:
						return -1
				endswitch
			endif
		endif
	endif
	
	Initialize(0)

	NVAR scrwpnt = root:Packages:ScreenSizer:scrwpnt
	NVAR scrhpnt = root:Packages:ScreenSizer:scrhpnt
	
	variable right, bottom
	
	variable ic,nt
	string theOne, theList, theType

	sprintf theType "WIN:%d" type
	theList=RemoveFromList("Procedure",WinList("*",";",theType))	
	if (!hidden)
		theList = RemoveHiddenWindows(theList)
	endif
	if (!ParamIsDefault(exclude))
		theList = RemoveFromList(exclude,theList)
	endif
	nt = ItemsInList(theList)
	if (nt==0)
		return 0
	endif
	for (ic=0;ic<nt;ic+=1)
		theOne=StringFromList(ic,theList)
		GetWindow $theOne wsize
		right=V_left + (width*scrwpnt)
		bottom=V_top + (height*scrhpnt)
		if (WinType(theOne)==0)
			MoveWindow/P=$theOne V_left,V_top,right,bottom
		else
			MoveWindow/W=$theOne V_left,V_top,right,bottom
		endif
	endfor
	
	return 0	
end

// Size Command Window

Static Function SizeCmdWindow(width, height)
	variable width, height
	
	Initialize(0)

	NVAR scrwpnt = root:Packages:ScreenSizer:scrwpnt
	NVAR scrhpnt = root:Packages:ScreenSizer:scrhpnt
	
	variable right, bottom

	GetWindow kwCmdHist wsize
	right=V_left + (width*scrwpnt)
	bottom=V_top + (height*scrhpnt)

	MoveWindow/C V_left,V_top,right,bottom

	return 0	
end

// Size Front Window Aspect Ratio

Static Function SizeFrontWindowAR(AR,[minormax,percent, width, height])
	variable AR, minormax, percent, width, height
		
	Initialize(0)

	if (ParamIsDefault(minormax))
		minormax = 1
	endif
	
	if (ParamIsDefault(percent))
		percent = 0
	endif
	
	if (ParamIsDefault(width))
		width = 0
	endif
	
	if (ParamIsDefault(height))
		height = 0
	endif
	
	string name = WinName(0,(1+2+4+16+64+128+4096))
	variable theType = WinType(name)
	
	SS_SizeAR(AR,minormax,width,height,percent,theType)

	return 0	
end

// Size a Window Aspect Ratio

Static Function SizeAWindowAR(AR,name,[minormax,percent,width,height])
	variable AR, minormax, percent, width, height
	string name
	
	variable theWinType=WinType(name)
	
	if (theWinType==0)
		if (strlen(WinList(name,"","WIN:128"))==0)
			return -1
		endif
	endif
		
	Initialize(0)

	if (ParamIsDefault(minormax))
		minormax = 1
	endif
	
	if (ParamIsDefault(percent))
		percent = 0
	endif
	
	if (ParamIsDefault(width))
		width = 0
	endif
	
	if (ParamIsDefault(height))
		height = 0
	endif
	
	variable theType = WinType("")
	
	SS_SizeAR(AR,minormax,width,height,percent,theType,name=name)

	return 0
	
end

// Size Windows Aspect Ratio
// type (optional): type number according to WinList and WinName, NOT WinType!
// kind (optional): graphs, tables, layouts, notebooks, panels, procedures, xops
// hidden (optional): 0 (default) - ignore hidden; 1 - also size hidden windows
// exclude (optional): string list of windows to exclude from size
// --> this command ignores panels, procedures, and XOP targets unless requested explicitly by type or kind
// --> this command always ignores the command window

Static Function SizeWindowsAR(AR,[minormax,percent,type,width,height,kind,hidden,exclude])
	variable AR, minormax, percent, type, hidden, width, height
	string kind, exclude
	
	if (ParamIsDefault(hidden))
		hidden=0
	endif
	
	if (ParamIsDefault(type) && ParamIsDefault(kind))
		type = 1 + 2 + 4 + 16
	else
		if (!ParamIsDefault(type))
			if (!((type & 2^0) || (type & 2^1) || (type & 2^2) || (type & 2^4) || (type & 2^6) || (type & 2^7) || (type & 2^12)))
				return -1
			endif
		else
			if (ParamIsDefault(kind))
				type = 1 + 2 + 4 + 16
			else
				strswitch(kind)
					case "graphs":
						type = 1
						break
					case "tables":
						type = 2
						break
					case "layouts":
						type = 4
						break
					case "notebooks":
						type = 16
						break
					case "panels":
						type = 64
						break
					case "procedures":
						type = 128
						break
					case "xops":
						type = 4096
						break
					default:
						return -1
				endswitch
			endif
		endif
	endif
	
	Initialize(0)

	NVAR scrwpnt = root:Packages:ScreenSizer:scrwpnt
	NVAR scrhpnt = root:Packages:ScreenSizer:scrhpnt
	
	variable itsType
	//right, bottom, width, height
	
	variable ic,nt
	string theOne, theList, theType

	sprintf theType "WIN:%d" type
	theList=RemoveFromList("Procedure",WinList("*",";",theType))	
	if (!hidden)
		theList = RemoveHiddenWindows(theList)
	endif
	if (!ParamIsDefault(exclude))
		theList = RemoveFromList(exclude,theList)
	endif
	nt = ItemsInList(theList)
	if (nt==0)
		return 0
	endif
		
	for (ic=0;ic<nt;ic+=1)
		theOne=StringFromList(ic,theList)	
		itsType = WinType(theOne)	
		SS_SizeAR(AR,minormax,width,height,percent,itsType,name=theOne)
	endfor
	
	return 0	
end

// Size Command Window Aspect Ratio

Static Function SizeCmdWindowAR(AR,[minormax,percent,width,height])
	variable AR, minormax, percent, width, height
		
	Initialize(0)

	if (ParamIsDefault(minormax))
		minormax = 1
	endif
	
	if (ParamIsDefault(percent))
		percent = 0
	endif
	
	if (ParamIsDefault(width))
		width = 0
	endif
	
	if (ParamIsDefault(height))
		height = 0
	endif
	
	SS_SizeAR(AR,minormax,width,height,percent,-1)

	return 0	
end

// SS_SizeAR ** internal function **

Function SS_SizeAR(AR,minormax,xwidth,xheight,percent,type,[name])
	variable AR, xwidth, xheight, minormax, percent, type
	string name

	NVAR scrwpnt = root:Packages:ScreenSizer:scrwpnt
	NVAR scrhpnt = root:Packages:ScreenSizer:scrhpnt
	
	variable width, height, right, bottom
	
	AR = abs(AR)

	if (ParamIsDefault(name))
		switch(type)
			case -1:
				GetWindow kwCmdHist wsize
				break		
			case 0:
				name = WinName(0,(1+2+4+16+64+128+4096))
				GetWindow $name wsize	
				break						
			default:
				name = WinName(0,type)
				GetWindow kwTopWin, wsize
				break
		endswitch
	else
		GetWindow $name wsize	
	endif
	
	if ((xheight!=0) || (xwidth!=0))
	
		// forcing height or width
	
		// height has priority!
		
		if (xheight!=0)
			height = xheight
			if (percent)
				width = AR*height
			else
				width = AR*height*scrhpnt/scrwpnt
			endif
		else
			width = xwidth
			if (percent)
				height = width/AR
			else
				height = width*scrwpnt/(AR*scrhpnt)			
			endif
		endif

		// above are % values
		
		percent = 1
		
	else

		// setting aspect ratio directly
		
		if (percent)
			height = (V_bottom - V_top)/scrhpnt
			width = (V_right - V_left)/scrwpnt
		else
			height = (V_bottom - V_top)
			width = (V_right - V_left)
		endif
		
		// determine whether maximum or minimum length is to dominate
		
		if (minormax)
			if (width>height)
				height = width/AR
			else
				width=height*AR
			endif
		else
			if (width<height)
				height = width/AR
			else
				width=height*AR
			endif
		endif
				
	endif
		
	if (percent)
		right=V_left + (width*scrwpnt)
		bottom=V_top + (height*scrhpnt)
	else
		right=V_left + width
		bottom=V_top + height
	endif
	
	switch(type)
		case -1:
			MoveWindow/C V_left,V_top,right,bottom
			break
		case 0:
			MoveWindow/P=$name V_left,V_top,right,bottom
			break
		default:
			MoveWindow/W=$name V_left,V_top,right,bottom
			break
	endswitch

	return 0

end

// Shift Front Window

Static Function ShiftFrontWindow(left, top)
	variable left, top

	string name = WinName(0,(1+2+4+16+64+128+4096))
	variable right, bottom	
	SetShiftPlace(name,left,top,right,bottom)

	if (WinType(name)==0)
		MoveWindow/P=$name left,top,right,bottom
	else	
		MoveWindow/W=$name left,top,right,bottom
	endif
	return 0
	
end

// Shift a Window

Static Function ShiftAWindow(left, top, name)
	variable left, top
	string name
	
	variable theWinType=WinType(name)
	
	if (theWinType==0)
		if (strlen(WinList(name,"","WIN:128"))==0)
			return -1
		endif
	endif	
	
	variable right, bottom
	SetShiftPlace(name,left,top,right,bottom)

	if (theWinType==0)
		MoveWindow/P=$name left,top,right,bottom
	else	
		MoveWindow/W=$name left,top,right,bottom
	endif
	return 0
	
end

// Shift Windows
// type (optional): type number according to WinList and WinName, NOT WinType!
// kind (optional): graphs, tables, layouts, notebooks, panels, procedures, xops
// hidden (optional): 0 (default) - ignore hidden; 1 - also shift hidden windows
// exclude (optional): string list of windows to exclude from shift
// dlr, dtp (optional): incremental multipliers to the shift in left-right or top-bottom
// --> this command ignores panels, procedures, and XOP targets unless requested explicitly by type or kind
// --> this command always ignores the command window

Static Function ShiftWindows(left, top, [type,kind,dlr,dtb,hidden,exclude])
	variable left, top, type, dlr, dtb, hidden
	string kind, exclude

	if (ParamIsDefault(hidden))
		hidden = 0
	endif
	
	if (ParamIsDefault(dlr))
		dlr = 0
	endif
	if (ParamIsDefault(dtb))
		dtb = 0
	endif
	
	if (ParamIsDefault(type) && ParamIsDefault(kind))
		type = 1 + 2 + 4 + 16
	else
		if (!ParamIsDefault(type))
			if (!((type & 2^0) || (type & 2^1) || (type & 2^2) || (type & 2^4) || (type & 2^6) || (type & 2^7) || (type & 2^12)))
				return -1
			endif
		else
			if (ParamIsDefault(kind))
				type = 1 + 2 + 4 + 16
			else
				strswitch(kind)
					case "graphs":
						type = 1
						break
					case "tables":
						type = 2
						break
					case "layouts":
						type = 4
						break
					case "notebooks":
						type = 16
						break
					case "panels":
						type = 64
						break
					case "procedures":
						type = 128
						break
					case "xops":
						type = 4096
						break
					default:
						return -1
				endswitch
			endif
		endif
	endif
	
	Initialize(0)
	
	NVAR scrleft = root:Packages:ScreenSizer:scrleft
	NVAR scrtop = root:Packages:ScreenSizer:scrtop
	NVAR scrright = root:Packages:ScreenSizer:scrright
	NVAR scrbottom = root:Packages:ScreenSizer:scrbottom

	NVAR scrwpnt = root:Packages:ScreenSizer:scrwpnt
	NVAR scrhpnt = root:Packages:ScreenSizer:scrhpnt
	
	variable right, bottom
	
	variable ic,nt,tleft=left, ttop=top
	string theOne, theList, theType

	sprintf theType "WIN:%d" type
	theList=RemoveFromList("Procedure",WinList("*",";",theType))	
	if (!hidden)
		theList = RemoveHiddenWindows(theList)
	endif
	if (!ParamIsDefault(exclude))
		theList = RemoveFromList(exclude,theList)
	endif
	theList=SortList(theList)
	nt = ItemsInList(theList)
	if (nt==0)
		return 0
	endif
	for (ic=0;ic<nt;ic+=1)
		left = tleft; top = ttop
		left+=dlr*ic
		top+=dtb*ic
		theOne=StringFromList(ic,theList)
		SetShiftPlace(theOne,left,top,right,bottom)
		if (WinType(theOne)==0)
			MoveWindow/P=$theOne left,top,right,bottom
		else
			MoveWindow/W=$theOne left,top,right,bottom
		endif
	endfor
	
	return 0
end

// Shift Command Window

Static Function ShiftCmdWindow(left, top)
	variable left, top
	
	variable right, bottom
	SetShiftPlace("",left,top,right,bottom)

	MoveWindow/C left,top,right,bottom
	return 0
	
end

// Make a Panel

Static Function MakePanel(left, top, width, height, strname, [other])
	variable left, top, width, height
	string strname, other
	
	Initialize(0)
	
	NVAR scrleft = root:Packages:ScreenSizer:scrleft
	NVAR scrtop = root:Packages:ScreenSizer:scrtop
	NVAR scrright = root:Packages:ScreenSizer:scrright
	NVAR scrbottom = root:Packages:ScreenSizer:scrbottom

	NVAR scrwpxl = root:Packages:ScreenSizer:scrwpxl
	NVAR scrhpxl = root:Packages:ScreenSizer:scrhpxl
	
	variable right, bottom
	string exeCmd

	if (left>=0)
		left=(scrwpxl*(scrleft + (scrright - scrleft)*left/100))
	else
		left=(scrwpxl*(scrright + (scrright - scrleft)*left/100)) - width
	endif
	if (top>=0)
		top=(scrhpxl*(scrtop + (scrbottom-scrtop)*top/100))
	else
		top=(scrhpxl*(scrbottom + (scrbottom-scrtop)*top/100)) - height
	endif
	right=left+width
	bottom=top+height
	
	if (ParamIsDefault(other))
		sprintf exeCmd "NewPanel/W=(%g,%g,%g,%g)as \"%s\"", left, top, right, bottom, strname
	else
		sprintf exeCmd "NewPanel/W=(%g,%g,%g,%g)%s as \"%s\"", left, top, right, bottom, other, strname
	endif
	execute/Q exeCmd
	return 0
	
end

// Make an External Panel

Static Function MakeExtPanel(side, offset, width, height, strname, [other])
	variable side, offset, width, height
	string strname, other
	
	Initialize(0)
	
	variable left, right, top, bottom
	string exeCmd

	switch(side)
		case 0:		
		case 1:
			left = offset
			top = 0
			right = width+offset
			bottom = height
			break
		default:
			left = 0
			top = offset
			right = width
			bottom = height+offset
			break
	endswitch
	
	if (ParamIsDefault(other))
		other = "/HOST=#"
	endif
	other += "/EXT=" + num2str(side)
	sprintf exeCmd "NewPanel/W=(%g,%g,%g,%g)%s as \"%s\"", left, top, right, bottom, other, strname
	execute/Q exeCmd
	return 0
	
end

// Save Window Coordinates

Static Function SaveWinCoords([name])
	string name
	
	string cdf = GetDataFolder(1), theCoords=""

	Initialize(0)
	
	NVAR scrwpnt = root:Packages:ScreenSizer:scrwpnt
	NVAR scrhpnt = root:Packages:ScreenSizer:scrhpnt

	if (ParamIsDefault(name))
		name = WinName(0,(1+2+4+16+64),1)
	endif
	switch(WinType(name))
		case 0:
			return -1
			break
		case 13:
			return -1
			break
		default:
			GetWindow $name wsize
			sprintf theCoords, "left=%g;top=%g;right=%g;bottom=%g;width=%g;height=%g", V_left, V_top, V_right, V_bottom, scrwpnt, scrhpnt
			SetWindow $name userData(ScreenSizer)=theCoords
			break
	endswitch
	return 0

end

// Restore Window Coordinates
// how:
//	-1 - restore LEFT, TOP and WIDTH, HEIGHT according to FIRST SCREEN
//	0 (default) - move LEFT, TOP according to NEW SCREEN and keep WIDTH, HEIGHT according to FIRST SCREEN
//	1 - move LEFT, TOP and WIDTH, HEIGHT according to NEW SCREEN
//	2 - move LEFT, TOP and WIDTH, HEIGHT according to NEW SCREEN and shift if outside screen (right, bottom) frame

Static Function RestoreWinCoords(how, [name])
	variable how
	string name
	
	string theSizes
	variable left, top, right, bottom, owidth, oheight, offset
	variable oscrwpnt, oscrhpnt
	
	if (ParamIsDefault(name))
		name = WinName(0,(1+2+4+16+64),1)
	endif

	theSizes = GetUserData(name,"","ScreenSizer")

	if (strlen(theSizes)==0)
		return -1
	endif
	
	left = NumberByKey("left",theSizes,"=")
	if (NumType(left)==2)
		return -1
	endif
	top = NumberByKey("top",theSizes,"=")
	right = NumberByKey("right",theSizes,"=")
	bottom = NumberByKey("bottom",theSizes,"=")
	oscrwpnt = NumberByKey("width",theSizes,"=")
	oscrhpnt = NumberByKey("height",theSizes,"=")
	owidth = right - left
	oheight = bottom - top
	
	Initialize(0)
	
	NVAR scrwpnt = root:Packages:ScreenSizer:scrwpnt
	NVAR scrhpnt = root:Packages:ScreenSizer:scrhpnt

	switch(WinType(name))
		case 0:
			return -1
		case 13:
			return -1
		case 7:	// panels always restore as either -1 or 0 or 2 ONLY
			switch(how)
				case -1:
					break
				case 0:
				case 1:
					left = (left/oscrwpnt)*scrwpnt
					top = (top/oscrhpnt)*scrhpnt
					right = left + owidth
					bottom = top + oheight
					break
				case 2:
					left = (left/oscrwpnt)*scrwpnt
					top = (top/oscrhpnt)*scrhpnt
					right = left + owidth
					if (right>scrwpnt*100)
						offset = scrwpnt*100 - right
						left = (left-offset) < 0 ? 0 : (left-offset)
						right = left + owidth
					endif
					bottom = top + oheight
					if (bottom>scrhpnt*100)
						offset = scrhpnt*100 - bottom
						top = (top-offset) < 0 ? 0 : (top-offset)
						bottom = top + oheight
					endif
					break
			endswitch
			break
		default:
			switch(how)
				case -1:
					break
				case 0:
					left = (left/oscrwpnt)*scrwpnt
					top = (top/oscrhpnt)*scrhpnt
					right = left + owidth
					bottom = top + oheight
					break
				case 1:
					left = (left/oscrwpnt)*scrwpnt
					top = (top/oscrhpnt)*scrhpnt
					right = left + (owidth/oscrwpnt)*scrwpnt
					bottom = top + (oheight/oscrhpnt)*scrhpnt
					break
				case 2:
					left = (left/oscrwpnt)*scrwpnt
					top = (top/oscrhpnt)*scrhpnt
					right = left + owidth
					if (right>scrwpnt*100)
						offset = scrwpnt*100 - right
						left = (left-offset) < 0 ? 0 : (left-offset)
						right = left + owidth
					endif
					bottom = top + oheight
					if (bottom>scrhpnt*100)
						offset = scrhpnt*100 - bottom
						top = (top-offset) < 0 ? 0 : (top-offset)
						bottom = top + oheight
					endif
					break
			endswitch
	endswitch
	MoveWindow/W=$name left, top, right, bottom
	return 0
end

Static Function SaveAllWindows()

	string theList = WinList("*",";","WIN:87"), theOne
	variable ic=0, nt = ItemsInList(theList)
	for(ic=nt;ic>=0;ic+=-1)
		theOne = StringFromList(ic,theList)
		SaveWinCoords(name=theOne)
	endfor
	return 0
end

Static Function RestoreAllWindows(how)
	variable how

	string theList = WinList("*",";","WIN:87"), theOne
	variable ic=0, nt=ItemsInList(theList)
	for(ic=nt-1;ic>=0;ic+=-1)
		theOne = StringFromList(ic,theList)
		RestoreWinCoords(how,name=theOne)
	endfor
	return 0
end

// Percentage of Screen Width left After Offset in piXels

Static Function pWaOx(offset)
	variable offset

	Initialize(0)
	
	NVAR scrleft = root:Packages:ScreenSizer:scrleft
	NVAR scrright = root:Packages:ScreenSizer:scrright

	NVAR scrwpxl = root:Packages:ScreenSizer:scrwpxl
	
	variable rtV
	
	 rtV = ((scrright - scrleft)*scrwpxl) - offset
	 
	 return (100*rtV/(rtV+offset))
	
end

// Percentage of Screen Height left After Offset in piXels

Static Function pHaOx(offset)
	variable offset

	Initialize(0)
	
	NVAR scrtop = root:Packages:ScreenSizer:scrtop
	NVAR scrbottom = root:Packages:ScreenSizer:scrbottom

	NVAR scrhpxl = root:Packages:ScreenSizer:scrhpxl
	
	variable rtV
	
	 rtV = ((scrbottom - scrtop)*scrhpxl) - offset
	 
	 return (100*rtV/(rtV+offset))
	
end

// Confirm Set
// checks that right >= left and bottom >= top

Static Function ConfirmSet(left,top,right,bottom)
	variable &left, &top, &right, &bottom
	
	variable mit, dit
	
	left = left < 0 ? 0 : left
	right = right > 100 ? 100 : right
	top = top < 0 ? 0 : top
	bottom = bottom > 100 ? 100 : bottom
	
	bottom = bottom <= top ? top : bottom
	right = right <= left ? left : right
	
	mit = left*right*top*bottom
	dit = mod(left,2)*mod(right,2)*mod(top,2)*mod(bottom,2)
	
	switch(mit)
		case 0:
			left = -1
			break
		case 1:
			left = 0
			break
		case 16:
			if (dit == 0)
				left = 0
			endif
	endswitch
	
	return 1
end

// Remove Hidden Windows

Static Function/S RemoveHiddenWindows(theList)
	string theList
	
	variable ic, nt
	string tList="", theOne=""
	
	nt = ItemsInList(theList)
	for (ic=0;ic<nt;ic+=1)
		theOne=StringFromList(ic,theList)
		GetWindow $theOne hide
		if (V_value==0)
			tList += theOne + ";"
		endif
	endfor
	
	return tList
end

// Set Shift Place
// consider negative left, top values as shifts from right, bottom

Static Function SetShiftPlace(name,left,top,right,bottom)
	string name
	variable &left, &top, &right, &bottom

	Initialize(0)
	
	NVAR scrleft = root:Packages:ScreenSizer:scrleft
	NVAR scrtop = root:Packages:ScreenSizer:scrtop
	NVAR scrright = root:Packages:ScreenSizer:scrright
	NVAR scrbottom = root:Packages:ScreenSizer:scrbottom

	NVAR scrwpnt = root:Packages:ScreenSizer:scrwpnt
	NVAR scrhpnt = root:Packages:ScreenSizer:scrhpnt

	if (strlen(name)==0)
		GetWindow kwCmdHist, wsize
	else
		GetWindow $name, wsize
	endif
	
	if (left>=0)
		left=(scrwpnt*(scrleft + (scrright - scrleft)*left/100))
		right = left + (V_right - V_left)
	else
		right=(scrwpnt*(scrright + (scrright - scrleft)*left/100))
		left = right - (V_right - V_left)
	endif
	if (top>=0)
		top=(scrhpnt*(scrtop + (scrbottom-scrtop)*top/100))
		bottom = top + (V_bottom - V_top)
	else
		bottom=(scrhpnt*(scrbottom + (scrbottom-scrtop)*top/100))
		top = bottom - (V_bottom - V_top)
	endif
	return 0
end

// Help Static Function

Static Function Help()

	if (!hasHelp)
		DoAlert 0, "NO HELP FOR THIS PACKAGE!"
		return 0
	endif

	if (strlen(thePackage)!=0)
		DisplayHelpTopic thePackage
	else
		DoAlert 0, "NO PACKAGE NAME FOR HELP!"
		return -1
	endif
	return 0
end

// Remove Me

Static Function RemoveMe([quiet])
	variable quiet
	
	if (ParamIsDefault(quiet) || (quiet == 0))
		DoAlert 1, KillMeRequest	
		if (V_flag == 2)
			return 0
		endif
	endif
	
	if (strlen(thePackageFolder)!=0)
		KillDataFolder/Z $thePackageFolder
	endif
	
	string theCmd
	if (strlen(thePackage)!=0)
		sprintf theCmd "DELETEINCLUDE \"%s\"" thePackage
		Execute/P/Z/Q theCmd
	endif
	if (strlen(theProcedureFile)!=0)
		sprintf theCmd "CloseProc/NAME=\"%s\"" theProcedureFile	
		Execute/P/Q theCmd
	endif
	
	return 0
end